# coding: utf-8

"""
    NetSuite REST Record API

    NetSuite REST Record API generated 2023-02-11 at 17:13:58 UTC for account 472052_SB1, user will@theapiguys.com with role Keap Integration.  # noqa: E501

    OpenAPI spec version: v1
    Contact: info@netsuite.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class Message(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'has_attachment': 'bool',
        'cc': 'str',
        'incoming': 'bool',
        'html_message': 'bool',
        'recipient_email': 'str',
        'author_email': 'str',
        'bcc': 'str',
        'subject': 'str',
        'emailed': 'bool',
        'last_modified_date': 'datetime',
        'external_id': 'str',
        'message_date': 'date',
        'message': 'str',
        'record': 'str',
        'record_type': 'str',
        'id': 'str',
        'time': 'str',
        'ref_name': 'str',
        'links': 'list[NsLink]',
        'recipient': 'OneOfmessageRecipient',
        'author': 'OneOfmessageAuthor',
        'activity': 'NsResource',
        'template': 'NsResource',
        'primary_recipient': 'OneOfmessagePrimaryRecipient',
        'transaction': 'NsResource',
        'entity': 'OneOfmessageEntity'
    }

    attribute_map = {
        'has_attachment': 'hasAttachment',
        'cc': 'cc',
        'incoming': 'incoming',
        'html_message': 'htmlMessage',
        'recipient_email': 'recipientEmail',
        'author_email': 'authorEmail',
        'bcc': 'bcc',
        'subject': 'subject',
        'emailed': 'emailed',
        'last_modified_date': 'lastModifiedDate',
        'external_id': 'externalId',
        'message_date': 'messageDate',
        'message': 'message',
        'record': 'record',
        'record_type': 'recordType',
        'id': 'id',
        'time': 'time',
        'ref_name': 'refName',
        'links': 'links',
        'recipient': 'recipient',
        'author': 'author',
        'activity': 'activity',
        'template': 'template',
        'primary_recipient': 'primaryRecipient',
        'transaction': 'transaction',
        'entity': 'entity'
    }

    def __init__(self, has_attachment=None, cc=None, incoming=None, html_message=None, recipient_email=None, author_email=None, bcc=None, subject=None, emailed=None, last_modified_date=None, external_id=None, message_date=None, message=None, record=None, record_type=None, id=None, time=None, ref_name=None, links=None, recipient=None, author=None, activity=None, template=None, primary_recipient=None, transaction=None, entity=None):  # noqa: E501
        """Message - a model defined in Swagger"""  # noqa: E501
        self._has_attachment = None
        self._cc = None
        self._incoming = None
        self._html_message = None
        self._recipient_email = None
        self._author_email = None
        self._bcc = None
        self._subject = None
        self._emailed = None
        self._last_modified_date = None
        self._external_id = None
        self._message_date = None
        self._message = None
        self._record = None
        self._record_type = None
        self._id = None
        self._time = None
        self._ref_name = None
        self._links = None
        self._recipient = None
        self._author = None
        self._activity = None
        self._template = None
        self._primary_recipient = None
        self._transaction = None
        self._entity = None
        self.discriminator = None
        if has_attachment is not None:
            self.has_attachment = has_attachment
        if cc is not None:
            self.cc = cc
        if incoming is not None:
            self.incoming = incoming
        if html_message is not None:
            self.html_message = html_message
        if recipient_email is not None:
            self.recipient_email = recipient_email
        if author_email is not None:
            self.author_email = author_email
        if bcc is not None:
            self.bcc = bcc
        if subject is not None:
            self.subject = subject
        if emailed is not None:
            self.emailed = emailed
        if last_modified_date is not None:
            self.last_modified_date = last_modified_date
        if external_id is not None:
            self.external_id = external_id
        if message_date is not None:
            self.message_date = message_date
        if message is not None:
            self.message = message
        if record is not None:
            self.record = record
        if record_type is not None:
            self.record_type = record_type
        if id is not None:
            self.id = id
        if time is not None:
            self.time = time
        if ref_name is not None:
            self.ref_name = ref_name
        if links is not None:
            self.links = links
        if recipient is not None:
            self.recipient = recipient
        if author is not None:
            self.author = author
        if activity is not None:
            self.activity = activity
        if template is not None:
            self.template = template
        if primary_recipient is not None:
            self.primary_recipient = primary_recipient
        if transaction is not None:
            self.transaction = transaction
        if entity is not None:
            self.entity = entity

    @property
    def has_attachment(self):
        """Gets the has_attachment of this Message.  # noqa: E501


        :return: The has_attachment of this Message.  # noqa: E501
        :rtype: bool
        """
        return self._has_attachment

    @has_attachment.setter
    def has_attachment(self, has_attachment):
        """Sets the has_attachment of this Message.


        :param has_attachment: The has_attachment of this Message.  # noqa: E501
        :type: bool
        """

        self._has_attachment = has_attachment

    @property
    def cc(self):
        """Gets the cc of this Message.  # noqa: E501

        Enter the email addresses you would like to receive a copy of this message. Separate multiple addresses with a comma or a semicolon. These email addresses will show to everyone who receives this message.  # noqa: E501

        :return: The cc of this Message.  # noqa: E501
        :rtype: str
        """
        return self._cc

    @cc.setter
    def cc(self, cc):
        """Sets the cc of this Message.

        Enter the email addresses you would like to receive a copy of this message. Separate multiple addresses with a comma or a semicolon. These email addresses will show to everyone who receives this message.  # noqa: E501

        :param cc: The cc of this Message.  # noqa: E501
        :type: str
        """

        self._cc = cc

    @property
    def incoming(self):
        """Gets the incoming of this Message.  # noqa: E501


        :return: The incoming of this Message.  # noqa: E501
        :rtype: bool
        """
        return self._incoming

    @incoming.setter
    def incoming(self, incoming):
        """Sets the incoming of this Message.


        :param incoming: The incoming of this Message.  # noqa: E501
        :type: bool
        """

        self._incoming = incoming

    @property
    def html_message(self):
        """Gets the html_message of this Message.  # noqa: E501


        :return: The html_message of this Message.  # noqa: E501
        :rtype: bool
        """
        return self._html_message

    @html_message.setter
    def html_message(self, html_message):
        """Sets the html_message of this Message.


        :param html_message: The html_message of this Message.  # noqa: E501
        :type: bool
        """

        self._html_message = html_message

    @property
    def recipient_email(self):
        """Gets the recipient_email of this Message.  # noqa: E501

        Enter the e-mail address of the recipient.  # noqa: E501

        :return: The recipient_email of this Message.  # noqa: E501
        :rtype: str
        """
        return self._recipient_email

    @recipient_email.setter
    def recipient_email(self, recipient_email):
        """Sets the recipient_email of this Message.

        Enter the e-mail address of the recipient.  # noqa: E501

        :param recipient_email: The recipient_email of this Message.  # noqa: E501
        :type: str
        """

        self._recipient_email = recipient_email

    @property
    def author_email(self):
        """Gets the author_email of this Message.  # noqa: E501

        Email address of sender.  # noqa: E501

        :return: The author_email of this Message.  # noqa: E501
        :rtype: str
        """
        return self._author_email

    @author_email.setter
    def author_email(self, author_email):
        """Sets the author_email of this Message.

        Email address of sender.  # noqa: E501

        :param author_email: The author_email of this Message.  # noqa: E501
        :type: str
        """

        self._author_email = author_email

    @property
    def bcc(self):
        """Gets the bcc of this Message.  # noqa: E501

        E-mail addresses you enter here receive a copy of this e-mail message, but these addresses do not show to the other recipients of the message. Separate multiple addresses with a comma or a semicolon.  # noqa: E501

        :return: The bcc of this Message.  # noqa: E501
        :rtype: str
        """
        return self._bcc

    @bcc.setter
    def bcc(self, bcc):
        """Sets the bcc of this Message.

        E-mail addresses you enter here receive a copy of this e-mail message, but these addresses do not show to the other recipients of the message. Separate multiple addresses with a comma or a semicolon.  # noqa: E501

        :param bcc: The bcc of this Message.  # noqa: E501
        :type: str
        """

        self._bcc = bcc

    @property
    def subject(self):
        """Gets the subject of this Message.  # noqa: E501

        Enter a subject or title for this email message. This subject is displayed on the messages subtab on the records of those receiving or sending the message.  # noqa: E501

        :return: The subject of this Message.  # noqa: E501
        :rtype: str
        """
        return self._subject

    @subject.setter
    def subject(self, subject):
        """Sets the subject of this Message.

        Enter a subject or title for this email message. This subject is displayed on the messages subtab on the records of those receiving or sending the message.  # noqa: E501

        :param subject: The subject of this Message.  # noqa: E501
        :type: str
        """

        self._subject = subject

    @property
    def emailed(self):
        """Gets the emailed of this Message.  # noqa: E501

        Check this box if you are pasting in an e-mail message.  # noqa: E501

        :return: The emailed of this Message.  # noqa: E501
        :rtype: bool
        """
        return self._emailed

    @emailed.setter
    def emailed(self, emailed):
        """Sets the emailed of this Message.

        Check this box if you are pasting in an e-mail message.  # noqa: E501

        :param emailed: The emailed of this Message.  # noqa: E501
        :type: bool
        """

        self._emailed = emailed

    @property
    def last_modified_date(self):
        """Gets the last_modified_date of this Message.  # noqa: E501


        :return: The last_modified_date of this Message.  # noqa: E501
        :rtype: datetime
        """
        return self._last_modified_date

    @last_modified_date.setter
    def last_modified_date(self, last_modified_date):
        """Sets the last_modified_date of this Message.


        :param last_modified_date: The last_modified_date of this Message.  # noqa: E501
        :type: datetime
        """

        self._last_modified_date = last_modified_date

    @property
    def external_id(self):
        """Gets the external_id of this Message.  # noqa: E501


        :return: The external_id of this Message.  # noqa: E501
        :rtype: str
        """
        return self._external_id

    @external_id.setter
    def external_id(self, external_id):
        """Sets the external_id of this Message.


        :param external_id: The external_id of this Message.  # noqa: E501
        :type: str
        """

        self._external_id = external_id

    @property
    def message_date(self):
        """Gets the message_date of this Message.  # noqa: E501


        :return: The message_date of this Message.  # noqa: E501
        :rtype: date
        """
        return self._message_date

    @message_date.setter
    def message_date(self, message_date):
        """Sets the message_date of this Message.


        :param message_date: The message_date of this Message.  # noqa: E501
        :type: date
        """

        self._message_date = message_date

    @property
    def message(self):
        """Gets the message of this Message.  # noqa: E501

        Paste or enter the text of the message in this field.  # noqa: E501

        :return: The message of this Message.  # noqa: E501
        :rtype: str
        """
        return self._message

    @message.setter
    def message(self, message):
        """Sets the message of this Message.

        Paste or enter the text of the message in this field.  # noqa: E501

        :param message: The message of this Message.  # noqa: E501
        :type: str
        """

        self._message = message

    @property
    def record(self):
        """Gets the record of this Message.  # noqa: E501


        :return: The record of this Message.  # noqa: E501
        :rtype: str
        """
        return self._record

    @record.setter
    def record(self, record):
        """Sets the record of this Message.


        :param record: The record of this Message.  # noqa: E501
        :type: str
        """

        self._record = record

    @property
    def record_type(self):
        """Gets the record_type of this Message.  # noqa: E501


        :return: The record_type of this Message.  # noqa: E501
        :rtype: str
        """
        return self._record_type

    @record_type.setter
    def record_type(self, record_type):
        """Sets the record_type of this Message.


        :param record_type: The record_type of this Message.  # noqa: E501
        :type: str
        """

        self._record_type = record_type

    @property
    def id(self):
        """Gets the id of this Message.  # noqa: E501


        :return: The id of this Message.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this Message.


        :param id: The id of this Message.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def time(self):
        """Gets the time of this Message.  # noqa: E501


        :return: The time of this Message.  # noqa: E501
        :rtype: str
        """
        return self._time

    @time.setter
    def time(self, time):
        """Sets the time of this Message.


        :param time: The time of this Message.  # noqa: E501
        :type: str
        """

        self._time = time

    @property
    def ref_name(self):
        """Gets the ref_name of this Message.  # noqa: E501


        :return: The ref_name of this Message.  # noqa: E501
        :rtype: str
        """
        return self._ref_name

    @ref_name.setter
    def ref_name(self, ref_name):
        """Sets the ref_name of this Message.


        :param ref_name: The ref_name of this Message.  # noqa: E501
        :type: str
        """

        self._ref_name = ref_name

    @property
    def links(self):
        """Gets the links of this Message.  # noqa: E501


        :return: The links of this Message.  # noqa: E501
        :rtype: list[NsLink]
        """
        return self._links

    @links.setter
    def links(self, links):
        """Sets the links of this Message.


        :param links: The links of this Message.  # noqa: E501
        :type: list[NsLink]
        """

        self._links = links

    @property
    def recipient(self):
        """Gets the recipient of this Message.  # noqa: E501


        :return: The recipient of this Message.  # noqa: E501
        :rtype: OneOfmessageRecipient
        """
        return self._recipient

    @recipient.setter
    def recipient(self, recipient):
        """Sets the recipient of this Message.


        :param recipient: The recipient of this Message.  # noqa: E501
        :type: OneOfmessageRecipient
        """

        self._recipient = recipient

    @property
    def author(self):
        """Gets the author of this Message.  # noqa: E501


        :return: The author of this Message.  # noqa: E501
        :rtype: OneOfmessageAuthor
        """
        return self._author

    @author.setter
    def author(self, author):
        """Sets the author of this Message.


        :param author: The author of this Message.  # noqa: E501
        :type: OneOfmessageAuthor
        """

        self._author = author

    @property
    def activity(self):
        """Gets the activity of this Message.  # noqa: E501


        :return: The activity of this Message.  # noqa: E501
        :rtype: NsResource
        """
        return self._activity

    @activity.setter
    def activity(self, activity):
        """Sets the activity of this Message.


        :param activity: The activity of this Message.  # noqa: E501
        :type: NsResource
        """

        self._activity = activity

    @property
    def template(self):
        """Gets the template of this Message.  # noqa: E501


        :return: The template of this Message.  # noqa: E501
        :rtype: NsResource
        """
        return self._template

    @template.setter
    def template(self, template):
        """Sets the template of this Message.


        :param template: The template of this Message.  # noqa: E501
        :type: NsResource
        """

        self._template = template

    @property
    def primary_recipient(self):
        """Gets the primary_recipient of this Message.  # noqa: E501


        :return: The primary_recipient of this Message.  # noqa: E501
        :rtype: OneOfmessagePrimaryRecipient
        """
        return self._primary_recipient

    @primary_recipient.setter
    def primary_recipient(self, primary_recipient):
        """Sets the primary_recipient of this Message.


        :param primary_recipient: The primary_recipient of this Message.  # noqa: E501
        :type: OneOfmessagePrimaryRecipient
        """

        self._primary_recipient = primary_recipient

    @property
    def transaction(self):
        """Gets the transaction of this Message.  # noqa: E501


        :return: The transaction of this Message.  # noqa: E501
        :rtype: NsResource
        """
        return self._transaction

    @transaction.setter
    def transaction(self, transaction):
        """Sets the transaction of this Message.


        :param transaction: The transaction of this Message.  # noqa: E501
        :type: NsResource
        """

        self._transaction = transaction

    @property
    def entity(self):
        """Gets the entity of this Message.  # noqa: E501


        :return: The entity of this Message.  # noqa: E501
        :rtype: OneOfmessageEntity
        """
        return self._entity

    @entity.setter
    def entity(self, entity):
        """Sets the entity of this Message.


        :param entity: The entity of this Message.  # noqa: E501
        :type: OneOfmessageEntity
        """

        self._entity = entity

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Message, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Message):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    NetSuite REST API

    NetSuite REST Record API generated 2023-01-18 at 19:40:11 UTC for account 472052, user will@theapiguys.com with role Keap Integration.  # noqa: E501

    OpenAPI spec version: v1
    Contact: info@netsuite.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class CustomerItemPricingElement(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'links': 'list[NsLink]',
        'price': 'float',
        'ref_name': 'str',
        'item': 'OneOfcustomerItemPricingElementItem',
        'level': 'NsResource'
    }

    attribute_map = {
        'links': 'links',
        'price': 'price',
        'ref_name': 'refName',
        'item': 'item',
        'level': 'level'
    }

    def __init__(self, links=None, price=None, ref_name=None, item=None, level=None):  # noqa: E501
        """CustomerItemPricingElement - a model defined in Swagger"""  # noqa: E501
        self._links = None
        self._price = None
        self._ref_name = None
        self._item = None
        self._level = None
        self.discriminator = None
        if links is not None:
            self.links = links
        if price is not None:
            self.price = price
        if ref_name is not None:
            self.ref_name = ref_name
        if item is not None:
            self.item = item
        if level is not None:
            self.level = level

    @property
    def links(self):
        """Gets the links of this CustomerItemPricingElement.  # noqa: E501


        :return: The links of this CustomerItemPricingElement.  # noqa: E501
        :rtype: list[NsLink]
        """
        return self._links

    @links.setter
    def links(self, links):
        """Sets the links of this CustomerItemPricingElement.


        :param links: The links of this CustomerItemPricingElement.  # noqa: E501
        :type: list[NsLink]
        """

        self._links = links

    @property
    def price(self):
        """Gets the price of this CustomerItemPricingElement.  # noqa: E501


        :return: The price of this CustomerItemPricingElement.  # noqa: E501
        :rtype: float
        """
        return self._price

    @price.setter
    def price(self, price):
        """Sets the price of this CustomerItemPricingElement.


        :param price: The price of this CustomerItemPricingElement.  # noqa: E501
        :type: float
        """

        self._price = price

    @property
    def ref_name(self):
        """Gets the ref_name of this CustomerItemPricingElement.  # noqa: E501


        :return: The ref_name of this CustomerItemPricingElement.  # noqa: E501
        :rtype: str
        """
        return self._ref_name

    @ref_name.setter
    def ref_name(self, ref_name):
        """Sets the ref_name of this CustomerItemPricingElement.


        :param ref_name: The ref_name of this CustomerItemPricingElement.  # noqa: E501
        :type: str
        """

        self._ref_name = ref_name

    @property
    def item(self):
        """Gets the item of this CustomerItemPricingElement.  # noqa: E501


        :return: The item of this CustomerItemPricingElement.  # noqa: E501
        :rtype: OneOfcustomerItemPricingElementItem
        """
        return self._item

    @item.setter
    def item(self, item):
        """Sets the item of this CustomerItemPricingElement.


        :param item: The item of this CustomerItemPricingElement.  # noqa: E501
        :type: OneOfcustomerItemPricingElementItem
        """

        self._item = item

    @property
    def level(self):
        """Gets the level of this CustomerItemPricingElement.  # noqa: E501


        :return: The level of this CustomerItemPricingElement.  # noqa: E501
        :rtype: NsResource
        """
        return self._level

    @level.setter
    def level(self, level):
        """Sets the level of this CustomerItemPricingElement.


        :param level: The level of this CustomerItemPricingElement.  # noqa: E501
        :type: NsResource
        """

        self._level = level

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(CustomerItemPricingElement, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, CustomerItemPricingElement):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



from netsuite import swagger_client


class RestClient:
    def __init__(self, netsuite):
        self.netsuite = netsuite
        self.configuration = swagger_client.Configuration()
        self.configuration.token = netsuite.storage.get_token(netsuite.app_name)
        self.configuration.token_refresh_hook = self.refresh_token
        self.configuration.app_name = netsuite.netsuite_app_name
        self.configuration.host = f"https://{self.configuration.app_name}.suitetalk.api.netsuite.com/services/rest/record/v1"
        self.api_client = swagger_client.ApiClient(configuration=self.configuration)
        self.contact_api = swagger_client.ContactApi(api_client=self.api_client)
        self.customer_api = swagger_client.CustomerApi(api_client=self.api_client)
        self.message_api = swagger_client.MessageApi(api_client=self.api_client)


    def refresh_token(self):
        self.configuration.token = self.netsuite.get_token()
        return self.configuration.token

class QueryClient:
    def __init__(self, netsuite):
        self.netsuite = netsuite
        self.configuration = swagger_client.Configuration()
        self.configuration.token = netsuite.storage.get_token(netsuite.app_name)
        self.configuration.token_refresh_hook = self.refresh_token
        self.configuration.app_name = netsuite.netsuite_app_name
        self.configuration.host = f"https://{self.configuration.app_name}.suitetalk.api.netsuite.com/services/rest/query/v1/suiteql?limit=500"
        self.query_api_client = swagger_client.ApiClient(configuration=self.configuration)
        self.query_api = swagger_client.QueryApi(api_client=self.query_api_client)

    def refresh_token(self):
        self.configuration.token = self.netsuite.get_token()
        return self.configuration.token






class NetsuiteException(Exception):
    pass


class NetsuiteForbiddenException(NetsuiteException):
    pass


class NetsuiteUnauthorizedException(NetsuiteException):
    pass


class NetsuiteTokenExpiredException(NetsuiteException):
    pass



# coding: utf-8

"""
    NetSuite REST API

    NetSuite REST Record API generated 2023-01-18 at 19:40:11 UTC for account 472052, user will@theapiguys.com with role Keap Integration.  # noqa: E501

    OpenAPI spec version: v1
    Contact: info@netsuite.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class NsError(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'type': 'str',
        'title': 'str',
        'status': 'int',
        'oerror_details': 'list[NsErrorOerrorDetails]'
    }

    attribute_map = {
        'type': 'type',
        'title': 'title',
        'status': 'status',
        'oerror_details': 'o:errorDetails'
    }

    def __init__(self, type=None, title=None, status=None, oerror_details=None):  # noqa: E501
        """NsError - a model defined in Swagger"""  # noqa: E501
        self._type = None
        self._title = None
        self._status = None
        self._oerror_details = None
        self.discriminator = None
        if type is not None:
            self.type = type
        if title is not None:
            self.title = title
        if status is not None:
            self.status = status
        if oerror_details is not None:
            self.oerror_details = oerror_details

    @property
    def type(self):
        """Gets the type of this NsError.  # noqa: E501

        A URI reference to the documentation about the problem type.  # noqa: E501

        :return: The type of this NsError.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this NsError.

        A URI reference to the documentation about the problem type.  # noqa: E501

        :param type: The type of this NsError.  # noqa: E501
        :type: str
        """

        self._type = type

    @property
    def title(self):
        """Gets the title of this NsError.  # noqa: E501

        A human-readable description of the problem type.  # noqa: E501

        :return: The title of this NsError.  # noqa: E501
        :rtype: str
        """
        return self._title

    @title.setter
    def title(self, title):
        """Sets the title of this NsError.

        A human-readable description of the problem type.  # noqa: E501

        :param title: The title of this NsError.  # noqa: E501
        :type: str
        """

        self._title = title

    @property
    def status(self):
        """Gets the status of this NsError.  # noqa: E501

        The HTTP status code generated by the server the request originates from.  # noqa: E501

        :return: The status of this NsError.  # noqa: E501
        :rtype: int
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this NsError.

        The HTTP status code generated by the server the request originates from.  # noqa: E501

        :param status: The status of this NsError.  # noqa: E501
        :type: int
        """

        self._status = status

    @property
    def oerror_details(self):
        """Gets the oerror_details of this NsError.  # noqa: E501

        An array containing one or more problem types.  # noqa: E501

        :return: The oerror_details of this NsError.  # noqa: E501
        :rtype: list[NsErrorOerrorDetails]
        """
        return self._oerror_details

    @oerror_details.setter
    def oerror_details(self, oerror_details):
        """Sets the oerror_details of this NsError.

        An array containing one or more problem types.  # noqa: E501

        :param oerror_details: The oerror_details of this NsError.  # noqa: E501
        :type: list[NsErrorOerrorDetails]
        """

        self._oerror_details = oerror_details

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(NsError, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, NsError):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



import json
import os

import click as click
from click import prompt

from netsuite.Netsuite import Netsuite
from netsuite.settings import api_settings, IN_MEMORY_STORAGE, JSON_STORAGE
from OpenSSL.SSL import FILETYPE_PEM
from OpenSSL.crypto import (dump_certificate, X509, X509Name, PKey, TYPE_RSA, X509Req, dump_privatekey, X509Extension)


@click.group()
def cli():
    """
    Simple CLI for managing Netsuite API Access
    """
    pass


@cli.command()
def generate_certificate():
    CN = prompt("Domain", hide_input=False)
    ORG = prompt("Organization", hide_input=False)
    ORG_UNIT = prompt("Department", hide_input=False)
    L = prompt("City", hide_input=False)
    ST = prompt("State", hide_input=False)
    C = prompt("Country", hide_input=False)
    EMAIL = prompt("Email", hide_input=False)

    NETSUITE_KEY_FILE = prompt("Netsuite Key FIle Name['./netsuite-key']: ", default=api_settings.NETSUITE_KEY_FILE)
    CERTIFICATE_FILE = './netsuite-certificate.pem'

    key = PKey()
    key.generate_key(TYPE_RSA, 4096)

    cert = X509()

    subject = cert.get_subject()
    subject.CN = CN
    subject.O = ORG
    subject.OU = ORG_UNIT
    subject.L = L
    subject.ST = ST
    subject.C = C
    subject.emailAddress = EMAIL

    cert.set_version(2)
    cert.set_issuer(subject)
    cert.set_subject(subject)
    cert.set_serial_number(int.from_bytes(os.urandom(16), byteorder="big"))
    # cert.set_serial_number(int(rand.bytes(16).encode('hex'), 16))
    cert.gmtime_adj_notBefore(0)
    cert.gmtime_adj_notAfter(31536000)
    cert.set_pubkey(key)
    cert.sign(key, 'sha256')

    with open(CERTIFICATE_FILE, 'wb+') as f:
        f.write(dump_certificate(FILETYPE_PEM, cert))
    with open(NETSUITE_KEY_FILE, 'wb+') as f:
        f.write(dump_privatekey(FILETYPE_PEM, key))


@cli.command()
def generate_client_config():
    client_id = prompt("What is your client id?", hide_input=False)
    cert_id = prompt("What is your Netsuite certificate id?", hide_input=False)
    cert_file = prompt("Name of Netsuite Key File['netsuite-key.pem']?", hide_input=False, default='netsuite-key.pem')

    # client_secret = prompt("What is your client secret?", hide_input=True)
    # redirect_url = prompt("Redirect URL", default=api_settings.REDIRECT_URL)
    netsuite_app_name = prompt("What is the netsuite application name?", hide_input=False)
    app_name = prompt("App Name", default=api_settings.APP_NAME)
    allow_none = prompt("Allow None", default=api_settings.ALLOW_NONE)
    use_datetime = prompt("Use Datetime", default=api_settings.USE_DATETIME)
    storage_class = prompt("Storage Class", default=api_settings.defaults.get('STORAGE_CLASS'),
                           type=click.Choice(api_settings.defaults.get('DEFAULT_STORAGE_CLASSES')),
                           show_choices=True)

    creds = {
        'CLIENT_ID': client_id,
        'NETSUITE_KEY_FILE': cert_file,
        'CERT_ID': cert_id,
        # 'CLIENT_SECRET': client_secret,
        # 'REDIRECT_URL': redirect_url,
        'NETSUITE_APP_NAME': netsuite_app_name,
        'APP_NAME': app_name,
        'ALLOW_NONE': allow_none,
        'USE_DATETIME': use_datetime,
        'STORAGE_CLASS': storage_class,
    }

    if storage_class == JSON_STORAGE:
        creds['JSON_STORAGE_PATH'] = prompt("Token Storage File", default=api_settings.JSON_STORAGE_PATH,
                                            type=click.Path(readable=True, writable=True))

    save_to_file = prompt("Save settings to file?", default='y', type=click.BOOL, show_choices=True)

    if save_to_file:
        file = prompt("Save settings to file?", default=api_settings.CREDENTIALS_PATH, type=click.File("w", ))
        file.write(json.dumps(creds, indent=4))
    else:
        click.echo(creds)

    return creds


@cli.command()
@click.option('--credentials-file', '--f', type=click.File('r'), default=api_settings.CREDENTIALS_PATH,
              prompt="Path to Credentials File")
@click.pass_context
def get_access_token(ctx, credentials_file):
    """OAuth flow for Netsuite to obtain an access and refresh token"""
    try:
        creds = json.load(credentials_file)
    except Exception as e:
        return

    creds['APP_NAME'] = prompt("What app is being used?", type=click.STRING, default=creds['APP_NAME'])

    netsuite = Netsuite(
        config=creds
    )
    netsuite.request_access_token()
    # auth_url = netsuite.get_authorization_url()
    # click.echo(f"Visit {auth_url}")
    # response_url = prompt("Paste the return url here")
    # query_string = dict(parse.parse_qsl(parse.urlsplit(response_url).query))
    # code = query_string.get('code')
    # if code:
    #     netsuite.request_access_token(code)

    if netsuite.token.access_token:
        if creds.get('STORAGE_CLASS') == IN_MEMORY_STORAGE:
            if prompt("You have chosen memory storage so nothing is saved here. Echo results?", default="y",
                      type=click.BOOL):
                click.echo(netsuite.token.__dict__)
            else:
                click.echo(f"Saved to {credentials_file}")


if __name__ == "__main__":
    cli()
